@@ -46,26 +46,30 @@ __license__ = "Apache 2.0"
 __copyright__ = "2021, ISCF committee"
 
 # Source URL
-__url__ = "https://github.com/riverlane/QHAL"
+__url__ = "https://gitlab.npl.co.uk/quantum-hardware-abstraction-layer/qhal"
 # Package Hosting URL
-__download_url__ = ""
+__download_url__ =  "https://pypi.org/project/hardware-abstraction-layer/#files"
+# Docs Hosting Url
+__docs_url__ = "https://quantum-hardware-abstraction-layer.gitlab-docs.npl.co.uk/qhal"
+
 
 contributors = [""]
 __contributors__ = ", ".join(contributors)
 __contributors_lines__ = "\n".join(contributors)
 __email__ = ""
 
-version_info = (0, 2, 0)
+version_info = (0, 3, 2)
 """Tuple[int, int, int] : version information
 The three components of the version:
-``major``, ``minor`` and ``micro``: Module level variable documented inline.
+k``major``, ``minor`` and ``micro``: Module level variable documented inline.
 """
 __version__ = ".".join(map(str, version_info[:3]))
 __version_description__ = ""
 
 
 __short_description__ = "Quantum HAL"
-__doc__ = ""
-__docs_url__ = ""
+__doc__ = """
+    UK effort to define a common Hardware Abstraction Layer for Quantum Computers.
+    """
 
 __platforms__ = ['']
