@@ -7,10 +7,7 @@ from .__about__ import (
     __doc__
 )
 
-from .hal import (HardwareAbstractionLayer,
-                  string_to_opcode,
-                  command_creator,
-                  Masks,
-                  Shifts)
+from .hal import (HardwareAbstractionLayer, command_creator, measurement_unpacker,
+                  QHALCommand, QHALCircuit, Opcode, HALMetadata)
 from .quantum_simulators import (IQuantumSimulator,
                                  ProjectqQuantumSimulator)
