@@ -1,4 +1,4 @@
-from abc import ABC, abstractclassmethod
+from abc import ABC, abstractmethod
 
 from numpy import uint64
 
@@ -8,9 +8,9 @@ class IQuantumSimulator(ABC):
     with the HAL.
     """
 
-    @abstractclassmethod
+    @abstractmethod
     def accept_command(
-        cls,
+        self,
         command: uint64
     ) -> uint64:
         """Performs required logic based on received commands, and returns
@@ -27,4 +27,3 @@ class IQuantumSimulator(ABC):
         uint64
             Result of a measurement command.
         """
-        pass
