@@ -12,7 +12,8 @@ from projectq.ops import (All, C, CNOT, DaggeredGate, H, Measure, R,
 from projectq.ops._basics import BasicGate, BasicRotationGate
 
 from . import IQuantumSimulator
-from ..hal import command_unpacker, string_to_opcode
+from ..hal._commands import QHALCommand, Opcode
+from ..hal._utils import binary_angle_conversion
 
 
 class SxGate(BasicGate):
@@ -123,35 +124,34 @@ class ProjectqQuantumSimulator(IQuantumSimulator):
 
         # assign projectq gate to each opcode
         self._parameterised_gate_dict = {
-            'CONTROL': C,
-            'R': R,
-            'RX': Rx,
-            'RY': Ry,
-            'RZ': Rz,
-            'PIXY': PiXY,
-            'PIYZ': PiYZ,
-            'PIZX': PiZX,
-            'PSWAP': Pswap,
-            'RXX': Rxx,
-            'RZZ': Rzz
+            Opcode.R: R,
+            Opcode.RX: Rx,
+            Opcode.RY: Ry,
+            Opcode.RZ: Rz,
+            Opcode.PIXY: PiXY,
+            Opcode.PIYZ: PiYZ,
+            Opcode.PIZX: PiZX,
+            Opcode.PSWAP: Pswap,
+            Opcode.RXX: Rxx,
+            Opcode.RZZ: Rzz
         }
 
         self._constant_gate_dict = {
             # SINGLE
-            'H': H,
-            'S': S,
-            'SQRT_X': SqrtX,
-            'T': T,
-            'X': X,
-            'Y': Y,
-            'Z': Z,
-            'INVT': DaggeredGate(T),
-            'INVS': DaggeredGate(S),
-            'SX': Sx,  # consecutive S and X gate, needed for RC
-            'SY': Sy,  # consecutive S and Y gate, needed for RC
+            Opcode.H: H,
+            Opcode.S: S,
+            Opcode.SQRT_X: SqrtX,
+            Opcode.T: T,
+            Opcode.X: X,
+            Opcode.Y: Y,
+            Opcode.Z: Z,
+            Opcode.INVT: DaggeredGate(T),
+            Opcode.INVS: DaggeredGate(S),
+            Opcode.SX: Sx,  # consecutive S and X gate, needed for RC
+            Opcode.SY: Sy,  # consecutive S and Y gate, needed for RC
             # DUAL
-            'CNOT': CNOT,
-            'SWAP': Swap
+            Opcode.CNOT: CNOT,
+            Opcode.SWAP: Swap
         }
         atexit.register(self.cleanup)
 
@@ -213,13 +213,13 @@ class ProjectqQuantumSimulator(IQuantumSimulator):
             else:  # multi qubit gate
                 if parameter_0 is not None:
                     gate(parameter_0) | (
-                        self._qubit_register[qubit_index_1],
-                        self._qubit_register[qubit_index_0]
+                        self._qubit_register[qubit_index_0],
+                        self._qubit_register[qubit_index_1]
                     )
                 else:
                     gate | (
-                        self._qubit_register[qubit_index_1],
-                        self._qubit_register[qubit_index_0]
+                        self._qubit_register[qubit_index_0],
+                        self._qubit_register[qubit_index_1]
                     )
 
             self._engine.flush()
@@ -246,8 +246,8 @@ class ProjectqQuantumSimulator(IQuantumSimulator):
         command: uint64
     ) -> uint64:
 
-        op, cmd_type, args, qubit_indexes = command_unpacker(command)
-        op_obj = string_to_opcode(op)
+        command_qhal = QHALCommand.from_binary(command)
+        op, args, qubit_indexes = command_qhal.opcode, command_qhal.args, command_qhal.qubits
 
         q_index_0 = qubit_indexes[0] + self.get_offset(0)
         q_index_1 = 0
@@ -255,17 +255,17 @@ class ProjectqQuantumSimulator(IQuantumSimulator):
             q_index_1 = qubit_indexes[1] + self.get_offset(1)
 
         for index in qubit_indexes:
-            assert index <= self._qubit_register_size, \
+            assert index < self._qubit_register_size, \
                 f"Qubit index {index} greater than register size " + \
                 f"({self._qubit_register_size})!"
 
-        if op == "START_SESSION":
+        if op == Opcode.START_SESSION:
             self._init_engine()
 
-        elif op == "STATE_PREPARATION_ALL":
+        elif op == Opcode.STATE_PREPARATION_ALL:
             self._init_qureg()
 
-        elif op == "STATE_PREPARATION":
+        elif op == Opcode.STATE_PREPARATION:
             if self._qubit_register is None:
                 self._init_qureg()
             elif q_index_0 in self._measured_qubits:
@@ -275,15 +275,19 @@ class ProjectqQuantumSimulator(IQuantumSimulator):
             else:
                 raise ValueError("Qubit already prepared!")
 
-        elif op == "END_SESSION":
+        elif op == Opcode.END_SESSION:
             self.cleanup()
 
-        elif op == "QUBIT_MEASURE":
+        elif op == Opcode.QUBIT_MEASURE:
 
             if q_index_0 in self._measured_qubits:
                 raise ValueError("Qubit already measured!")
 
-            # This measures a single qubit at the time.
+            # In the ProjectQ simulator we implement this via a rotation and then measurement.
+            arg0 = binary_angle_conversion(args[0])
+            arg1 = binary_angle_conversion(args[1])
+            Rz(-arg1) | self._qubit_register[q_index_0]
+            Ry(-arg0) | self._qubit_register[q_index_0]
             Measure | self._qubit_register[q_index_0]
             self._engine.flush()
 
@@ -301,40 +305,46 @@ class ProjectqQuantumSimulator(IQuantumSimulator):
                 | measurement
             )
 
-        elif op.split("_")[0] == "PAGE":
-            self._offset_registers[int(op.split("_")[3])] = qubit_indexes[0]
+        elif op == Opcode.PAGE_SET_QUBIT_0:
+            self._offset_registers[0] = qubit_indexes[0]
 
-        elif op == "ID":
+        elif op == Opcode.PAGE_SET_QUBIT_1:
+            self._offset_registers[1] = qubit_indexes[0]
+
+        elif op == Opcode.ID:
             pass
 
-        elif op_obj.param == "PARAM":
+        elif op.has_param():
             if q_index_0 in self._measured_qubits:
                 raise ValueError("Qubit requires re-preparation!")
 
-            angle = args[-1] * (2 * np.pi) / 65536
+            arg0 = binary_angle_conversion(args[0])
+            arg1 = binary_angle_conversion(args[1])
             gate = self._parameterised_gate_dict[op]
-            if cmd_type == "SINGLE":
-                self.apply_gate(gate, q_index_0, parameter_0=angle)
-            else:
+            if op.is_dual():
                 self.apply_gate(
                     gate,
                     qubit_index_0=q_index_0,
                     qubit_index_1=q_index_1,
-                    parameter_0=angle
+                    parameter_0=arg0,
+                    parameter_1=arg1
                 )
+            else:
+                self.apply_gate(gate, q_index_0,
+                                parameter_0=arg0, parameter_1=arg1)
 
-        elif op_obj.param == "CONST":
+        elif not op.has_param():
             if q_index_0 in self._measured_qubits:
                 raise ValueError("Qubit requires re-preparation!")
 
             gate = self._constant_gate_dict[op]
-            if cmd_type == "SINGLE":
-                self.apply_gate(gate, q_index_0)
-            else:
+            if op.is_dual():
                 self.apply_gate(
                     gate,
                     qubit_index_0=q_index_0,
                     qubit_index_1=q_index_1
                 )
+            else:
+                self.apply_gate(gate, q_index_0)
         else:
             raise TypeError(f"{op} is not a recognised opcode!")
